﻿--[[----------------------------------------------------------------------
	ChatNotifier Module - Part of VanasKoS
modifies the ChatMessage if a player speaks whom is on your hatelist
------------------------------------------------------------------------]]

local L = AceLibrary("AceLocale-2.2"):new("VanasKoSChatNotifier");

L:RegisterTranslations("enUS", function() return {
	["[HateList: %s] %s"] = true,
	
	["Chat Modifications"] = true,
	["Modifies the Chat Window for Hate/Nicelist Entries."] = true,
	["Enabled"] = true,
	["Hatelist Color"] = true,
	["Sets the Foreground Color for Hatelist Entries"] = true,
	["Nicelist Color"] = true,
	["Sets the Foreground Color for Nicelist Entries"] = true,
	["Modify only my Entries"] = true,
	["Modifies the Chat only for your Entries"] = true,
} end);

L:RegisterTranslations("deDE", function() return {
	["[HateList: %s] %s"] = "[Hassliste: %s] %s",
	
	["Chat Modifications"] = "Chat Modifikationen",
	["Modifies the Chat Window for Hate/Nicelist Entries."] = "Modifiziert das Chatfenster Spieler auf der Hass- und Nette-Leuteliste",
	["Enabled"] = "Aktiviert",
	["Hatelist Color"] = "Farbe für Hassliste",
	["Sets the Foreground Color for Hatelist Entries"] = "Setzt die Vordergrundfarbe für Einträge der Hassliste",
	["Nicelist Color"] = "Farbe für Nette-Leuteliste",
	["Sets the Foreground Color for Nicelist Entries"] = "Setzt die Hintergrundfarbe für Einträge der Nette-Leuteliste",
	["Modifies the Chat only for your Entries"] = "Nur für meine Einträge Chat ändern",
} end);

L:RegisterTranslations("frFR", function() return {
	["[HateList: %s] %s"] = "[Liste noire: %s] %s",
	
	["Chat Modifications"] = "Modifications du chat",
	["Modifies the Chat Window for Hate/Nicelist Entries."] = "Modifie la fenêtre du chat pour les entrées des listes blanche/noire",
	["Enabled"] = "Actif",
	["Hatelist Color"] = "Couleur pour liste noire",
	["Sets the Foreground Color for Hatelist Entries"] = "Choisir la couleur de texte pour les entrées de la liste noire",
	["Nicelist Color"] = "Couleur pour liste blanche",
	["Sets the Foreground Color for Nicelist Entries"] = "Choisir la couleur de texte pour les entrées de la liste blanche",
	["Modify only my Entries"] = "Modifier seulement mes entrées",
	["Modifies the Chat only for your Entries"] = "Modifie le chat seulement pour vos entrées",
} end);

L:RegisterTranslations("koKR", function() return {
	["[HateList: %s] %s"] = "[악인명부: %s] %s",
	
	["Chat Modifications"] = "대화창 변경",
	["Modifies the Chat Window for Hate/Nicelist Entries."] = "악인/호인에 대해 대화창을 변경합니다.",
	["Enabled"] = "사용",
	["Hatelist Color"] = "악인명부 색상",
	["Sets the Foreground Color for Hatelist Entries"] = "싫어하는 사람에 대한 전경 색상을 설정합니다.",
	["Nicelist Color"] = "호인명부 색상",
	["Sets the Foreground Color for Nicelist Entries"] = "좋아하는 사람에 대한 전경 색상을 설정합니다.",
	["Modify only my Entries"] = "내 명부만 변경",
	["Modifies the Chat only for your Entries"] = "당신의 명부에 대한 대화만 변경합니다.",
} end);

L:RegisterTranslations("esES", function() return {
	["[HateList: %s] %s"] = "[Odiados: %s] %s",
	
	["Chat Modifications"] = "Modificaciones de Chat",
	["Modifies the Chat Window for Hate/Nicelist Entries."] = "Modifica la ventana de chat para las entradas de Simpáticos/Odiados",
	["Enabled"] = "Activado",
	["Hatelist Color"] = "Color de Odiados",
	["Sets the Foreground Color for Hatelist Entries"] = "Establece el color de las entradas de Odiados",
	["Nicelist Color"] = "Color de Simpáticos",
	["Sets the Foreground Color for Nicelist Entries"] = "Establece el color de las entradas de Simpáticos",
	["Modify only my Entries"] = "Modificar sólo mis entradas",
	["Modifies the Chat only for your Entries"] = "Modifica el chat sólo para tus entradas",
} end);

VanasKoSChatNotifier = VanasKoS:NewModule("ChatNotifier");

local VanasKoSChatNotifier = VanasKoSChatNotifier;

function VanasKoSChatNotifier:OnInitialize()
	VanasKoS:RegisterDefaults("ChatNotifier", "profile", {
		Enabled = true,
		OnlyMyEntries = false,
		
		HateListColorR = 1.0,
		HateListColorG = 0.0,
		HateListColorB = 0.0,
		
		NiceListColorR = 0.0,
		NiceListColorG = 1.0,
		NiceListColorB = 0.0,
	});
	self.db = VanasKoS:AcquireDBNamespace("ChatNotifier");

	VanasKoSGUI:AddConfigOption("ChatNotifier", {
		type = 'group',
		name = L["Chat Modifications"],
		desc = L["Modifies the Chat Window for Hate/Nicelist Entries."],
		args = {
			enabled = {
				type = 'toggle',
				name = L["Enabled"],
				desc = L["Enabled"],
				order = 1,
				set = function() VanasKoSChatNotifier.db.profile.Enabled = not VanasKoSChatNotifier.db.profile.Enabled; VanasKoS:ToggleModuleActive("ChatNotifier", VanasKoSChatNotifier.db.profile.Enabled); end,
				get = function() return VanasKoSChatNotifier.db.profile.Enabled end,
			},
			hateListColor = {
				type = 'color',
				name = L["Hatelist Color"],
				desc = L["Sets the Foreground Color for Hatelist Entries"],
				order = 2,
				get = function() return VanasKoSChatNotifier:GetColor("HateListColor") end,
				set = function(r, g, b, a) VanasKoSChatNotifier:SetColor("HateListColor", r, g, b); end,
				hasAlpha = false,
			},
			niceListColor = {
				type = 'color',
				name = L["Nicelist Color"],
				desc = L["Sets the Foreground Color for Nicelist Entries"],
				order = 3,
				get = function() return VanasKoSChatNotifier:GetColor("NiceListColor") end,
				set = function(r, g, b, a) VanasKoSChatNotifier:SetColor("NiceListColor", r, g, b); end,
				hasAlpha = false,
			},
			modifyOnlyMyEntries = {
				type = 'toggle',
				name = L["Modify only my Entries"],
				desc = L["Modifies the Chat only for your Entries"],
				order = 4,
				set = function() VanasKoSChatNotifier.db.profile.OnlyMyEntries = not VanasKoSChatNotifier.db.profile.OnlyMyEntries; end,
				get = function() return VanasKoSChatNotifier.db.profile.OnlyMyEntries; end,
			}
		}
	});
end

function VanasKoSChatNotifier:OnEnable()
	if(not self.db.profile.Enabled) then
		return nil;
	end
	self:Hook("ChatFrame_MessageEventHandler", true);
end

function VanasKoSChatNotifier:OnDisable()
end

local function SetChatTypeInfo(type, r,g,b)
	ChatTypeInfo[type].r = r;
	ChatTypeInfo[type].g = g;
	ChatTypeInfo[type].b = b;
end

-- BUG: channel message don't get colored (trade, general!)
-- /script arg1 = "test"; arg2 = "Cyria"; arg3= "Common"; arg4="5. General - Stormwind City"; arg8 = 1; arg9 = "General"; VanasKoSChatNotifier:ChatFrame_MessageEventHandler("CHAT_MSG_CHANNEL")
function VanasKoSChatNotifier:ChatFrame_MessageEventHandler(event)
	-- if switched to disabled, remove hook on first intercept
	if(not self.db.profile.Enabled) then
		local ret = self.hooks["ChatFrame_MessageEventHandler"](event);
		self:Unhook("ChatFrame_MessageEventHandler");
		return ret;
	end
	
	-- get the event type and backup the colordata for this event
	-- only handle CHAT_MSG events
	if(not (event == "CHAT_MSG_CHANNEL" or 
			event == "CHAT_MSG_EMOTE" or 
			event == "CHAT_MSG_GUILD" or 
			event == "CHAT_MSG_PARTY" or 
			event == "CHAT_MSG_RAID" or 
			event == "CHAT_MSG_RAID_LEADER" or 
			event == "CHAT_MSG_SAY" or 
			event == "CHAT_MSG_WHISPER" or
			event == "CHAT_MSG_OFFICER") ) then
		return self.hooks["ChatFrame_MessageEventHandler"](event);
	else
		local backupInfo = { };
		local type = strsub(event, 10);
		-- temporary change the color blizzards chatframe uses
		if(event == "CHAT_MSG_CHANNEL") then
			if(not ChatTypeInfo["CHANNEL" .. arg8]) then
				return self.hooks["ChatFrame_MessageEventHandler"](event);
			end
			backupInfo.r = ChatTypeInfo["CHANNEL" .. arg8].r;
			backupInfo.g = ChatTypeInfo["CHANNEL" .. arg8].g;
			backupInfo.b = ChatTypeInfo["CHANNEL" .. arg8].b;
			backupInfo.id = ChatTypeInfo["CHANNEL" .. arg8].id;
		else
			if(not ChatTypeInfo[type]) then
				return self.hooks["ChatFrame_MessageEventHandler"](event);
			end
			backupInfo.r = ChatTypeInfo[type].r;
			backupInfo.g = ChatTypeInfo[type].g;
			backupInfo.b = ChatTypeInfo[type].b;
			backupInfo.id = ChatTypeInfo[type].id;
		end
		
		
		local reason, creator, owner = VanasKoS:IsOnList("HATELIST", arg2);
		if(reason ~= nil) then
			local r,g,b = VanasKoSChatNotifier:GetColor("HateListColor");

			if(self.db.profile.OnlyMyEntries) then
				if(owner == nil) then
					if(event == "CHAT_MSG_CHANNEL") then
						SetChatTypeInfo("CHANNEL" .. arg8, r, g, b);
					else
						SetChatTypeInfo(type, r, g, b);
					end
				end
			else
				local r,g,b = VanasKoSChatNotifier:GetColor("HateListColor");
				if(event == "CHAT_MSG_CHANNEL") then
					SetChatTypeInfo("CHANNEL" .. arg8, r, g, b);
				else
					SetChatTypeInfo(type, r, g, b);
				end
			end
			--arg1 = format(L["[HateList: %s] %s"], VanasKoS:IsOnList("HATELIST", arg2), arg1);
		else
			local reason, creator, owner = VanasKoS:IsOnList("NICELIST", arg2);
			if(reason ~= nil) then
				local r,g,b = VanasKoSChatNotifier:GetColor("NiceListColor");

				if(self.db.profile.OnlyMyEntries) then
					if(owner == nil) then
						if(event == "CHAT_MSG_CHANNEL") then
							SetChatTypeInfo("CHANNEL" .. arg8, r, g, b);
						else
							SetChatTypeInfo(type, r, g, b);
						end
					end
				else
					if(event == "CHAT_MSG_CHANNEL") then
						SetChatTypeInfo("CHANNEL" .. arg8, r, g, b);
					else
						SetChatTypeInfo(type, r, g, b);
					end
				end
			end
		end
		
		local ret = self.hooks["ChatFrame_MessageEventHandler"](event);
		if(event == "CHAT_MSG_CHANNEL") then
			SetChatTypeInfo("CHANNEL" .. arg8, backupInfo.r, backupInfo.g, backupInfo.b);
			ChatTypeInfo["CHANNEL" .. arg8].id = backupInfo.id;
		else
			SetChatTypeInfo(type, backupInfo.r, backupInfo.g, backupInfo.b);
			ChatTypeInfo[type].id = backupInfo.id;
		end
		return ret;
	end
end

function VanasKoSChatNotifier:GetColor(which)
	return self.db.profile[which .. "R"], self.db.profile[which .. "G"], self.db.profile[which .. "B"], self.db.profile[which .. "A"];
end

function VanasKoSChatNotifier:SetColor(which, r, g, b)
	self.db.profiles[which .. "R"] = r;
	self.db.profiles[which .. "G"] = g;
	self.db.profiles[which .. "B"] = g;
end
